
********************************************
* File Locations                           *
********************************************

local excel_file "Levin et al. supplemental data 28july2020.xlsx"

*********************************************
* Part I: Benchmark Metaregression Analysis *
*********************************************

* Import age-specific IFR data

clear 
graph drop _all
import excel using "`excel_file'", sheet("Benchmark Studies") firstrow case(lower)
sort study median_age

* Exclude observations with median age < 21 years

drop if median_age < 35

* Create indicator variables

gen Belgium_study = cond(strpos(study,"Belgium")>0,1,0)
gen Geneva_study = cond(strpos(study,"Geneva")>0,1,0)
gen Indiana_study = cond(strpos(study,"Indiana")>0,1,0)
gen NY_study = cond(strpos(study,"New York")>0,1,0)
gen Spain_study = cond(strpos(study,"Spain")>0,1,0)
gen Sweden_study = cond(strpos(study,"Sweden")>0,1,0)
local studylist Belgium_study Geneva_study Indiana_study NY_study Spain_study Sweden_study

egen istudy = group(study)
by study: gen study_id = 10*istudy + _n

* Compute each standard error using the 95% confidence interval and Gaussian thresholds 
* i.e., 95% confidence interval = mean +/- 1.96*SE

gen ifr_se = (ifr_ci95_high-ifr_ci95_low)/3.84

* Generate log(ifr) and compute standard error using the delta method
* i.e., SE(log(xhat)) = SE(xhat)/xhat

gen log_ifr = log(ifr)
gen log_ifr_se = ifr_se/ifr
gen log_ifr_precision = 1/log_ifr_se^2

* List observations

sort study_id
list study_id study median_age log_ifr log_ifr_se
sort median_age

* Run benchmark metaregression

metareg log_ifr median_age, wsse(log_ifr_se) tau2test 
predict ueffects, ustandard
qnorm ueffects, mlabel(study_id) name(ifr_meta_qnorm)

* Append data with sequential series of median_age 
* for generating smooth predicted curve in plots

local ncount = _N
set obs `=`ncount' + 61'
forvalues ix = 1/61 {
  quietly replace median_age = 34 + `ix' if _n == `=`ncount' + `ix''
  }

* Use the predict function of metareg to generate fitted values,
* confidence intervals, and prediction intervals for log model
 
predict fit if missing(study)
predict stdp if missing(study), stdp
predict stdf if missing(study), stdf
predict xbu if missing(study), xbu
quietly gen conf_low = fit - 1.96*stdp 
quietly gen conf_high = fit + 1.96*stdp
quietly gen pred_low = fit - 1.96*stdf
quietly gen pred_high = fit + 1.96*stdf
quietly gen log_ifr_study = log(ifr)
separate log_ifr_study, by(study) veryshortlabel

* Compute estimates and intervals for level of IFR in percent

gen ifr_study = ifr
separate ifr_study, by(study) veryshortlabel

gen expfit = exp(fit) if missing(study)
gen exp_conf_low = exp(conf_low) if missing(study)
gen exp_conf_high = exp(conf_high) if missing(study)
gen exp_pred_low = exp(pred_low) if missing(study)
gen exp_pred_high = exp(pred_high) if missing(study)

gen exp_conf_hitop = min(exp_conf_high,21) if missing(study)
gen exp_pred_hitop = min(exp_pred_high,21) if missing(study)


* Produce chart of meta-regression in logs

local bot_age = 35
local top_age = 95
twoway line fit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	      legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) ///
	|| rarea pred_low pred_high median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(2 "Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea conf_low conf_high median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(3 "Confidence Interval")) fcolor(purple%30) lcolor(blue) ///
    || scatter log_ifr_study? median_age ///
	      if inrange(median_age,`bot_age',`top_age'), ///
	      msymbol(O D S T) mcolor(cranberry black green blue orange) ///
   	   legend(pos(11) ring(0) col(3) colfirst order(1 3 2 4 5 6 7 8 9) region(lwidth(thin) fcolor(none))) ///
	   xlabel(35 45 55 65 75 85 95) ylabel(-5 -2.5 0 2.5 5 7.5) xtitle("Median Age",height(5)) ///
	   name(ifr_meta_log_plot) title("Logarithm of IFR", placement(ne) size(medsmall))

* Produce chart of meta-regression in levels

local bot_age = 45
local top_age = 85
sort median_age
twoway line expfit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	   legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) ///
	|| rarea exp_pred_low exp_pred_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
       legend(label(2 "95% Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea exp_conf_low exp_conf_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
       legend(label(3 "95% Confidence Interval")) fcolor(purple%30) lcolor(blue) ///
    || scatter ifr_study? median_age if inrange(median_age,`bot_age',`top_age'), ///
	   msymbol(O D S T) mcolor(cranberry black green blue orange) ///
   	   legend(pos(11) ring(0) col(1) order(1 3 2 4 5 6 7 8 9)) ///
	   xlabel(45 45 55 65 75 85) ylabel(0 3 6 9 12 15 18 21) ///
	   xtitle("Median Age",height(5)) title("Percent", placement(ne) size(medsmall)) ///
	   name(ifr_meta_plot)

* Save prediction intervals for comparison with other studies

keep study median_age fit conf_low conf_high pred_low pred_high ///
     expfit exp_conf_low exp_conf_high exp_pred_low exp_pred_high 
drop if !missing(study)
save "metareg_predictions.dta", replace

*****************************************************************
* Part II: Comparison of Benchmark Predictions with CDC Studies *
*****************************************************************

* Import data from excel

clear
import excel study=A agegroup=B median_age=C population=D infection_rate=E infrate_ci95_low=F infrate_ci95_high=G /// 
             deaths=H ifr=I ifr_ci95_low=J ifr_ci95_high=K ///
using "`excel_file'", sheet("Other U.S. Studies") cellrange(A3:K37)
destring infrate_ci95_low infrate_ci95_high ifr_ci95_low ifr_ci95_high, force replace
drop if missing(study) & missing(agegroup) & missing(population) & missing(infection_rate)
drop if median_age < 35

* Compute estimates and confidence intervals for number of infections

gen infected = 0.01 * infection_rate * population
gen infected_ci95_low = 0.01 * infrate_ci95_low * population
gen infected_ci95_high =  0.01 * infrate_ci95_high * population

* Generate age-specific IFR and 95% confidence intervals

gen ifr_check = ifr - (100 * deaths / infected)
list if abs(ifr_check)>0.001 

quietly gen ifr_se = (ifr_ci95_high-ifr_ci95_low)/3.84 if ifr_ci95_low
list study agegroup median_age ifr ifr_se 

* Create study-specific 

quietly gen ifr_other = ifr
separate ifr_other, by(study) veryshortlabel
quietly gen log_ifr_other = log(ifr) 
separate log_ifr_other, by(study) veryshortlabel

* List observations

sort study median_age
list study median_age ifr ifr_se

* Merge with metareg predictions

merge 1:1 study median_age using "Metareg_predictions.dta" 
gen exp_conf_hitop = min(exp_conf_high,24) if missing(study)
gen exp_pred_hitop = min(exp_pred_high,24) if missing(study)
sort median_age

* Show comparison in levels

local bot_age = 45
local top_age = 85
twoway line expfit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	      legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) ///
	|| rarea exp_pred_low exp_pred_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(2 "Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea exp_conf_low exp_conf_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(3 "Confidence Interval")) fcolor(purple%30) lcolor(blue) /// 
    || scatter ifr_other? median_age if inrange(median_age,`bot_age',`top_age'), ///
	   msymbol(T Oh Sh Dh Th o sh dh th) ///
	   mcolor(emerald lavender navy mint magenta olive orange navy black) ///
   	   legend(pos(11) ring(0) region(lwidth(thin) fcolor(none)) ///
	   order(1 3 2 4 5 6 7 8 9 10 11 12) ///
	     colgap(2) rowgap(1.1) cols(1) colfirst) xtitle("Median Age",height(5)) ///  
	   xlabel(45 50 55 60 65 70 75 80 85) ylabel(0 3 6 9 12 15 18 21 24) ///
	   title("Percent", placement(ne) size(medsmall)) name(CDC_comparison_chart)


********************************************************************
* Part III: Comparison of Benchmark Predictions with Other Studies *
********************************************************************

* Import data from excel
clear
import excel study=A agegroup=B median_age=C population=D infections=E deaths=F infection_rate=H ///
             infrate_ci95_low=I infrate_ci95_high=J ifr=L ifr_ci95_low=M ifr_ci95_high=N ///
       using "`excel_file'", sheet("Small-Scale Studies") cellrange(A3:N14) 
destring infrate_ci95_low infrate_ci95_high ifr_ci95_low ifr_ci95_high, force replace
save "temp_store.dta", replace
clear
import excel study=A agegroup=B median_age=C population=D infections=E deaths=F infection_rate=H ///
             infrate_ci95_low=I infrate_ci95_high=J ifr=L ifr_ci95_low=M ifr_ci95_high=N ///
       using "`excel_file'", sheet("Track&Trace Countries") cellrange(A3:N23) 
destring infrate_ci95_low infrate_ci95_high ifr_ci95_low ifr_ci95_high, force replace
append using "temp_store.dta"
drop if missing(study) & missing(agegroup) & missing(population) & missing(infection_rate)
drop if median_age < 35

* Compute estimates and confidence intervals for number of infections

gen infected = 0.01 * infection_rate * population
gen infected_ci95_low = 0.01 * infrate_ci95_low * population
gen infected_ci95_high =  0.01 * infrate_ci95_high * population

* Generate age-specific IFR and 95% confidence intervals

gen ifr_check = ifr - (100 * deaths / infected)
list if abs(ifr_check)>0.001 

gen ifr_se = (ifr_ci95_high-ifr_ci95_low)/3.84

gen traceflag = strpos(study,"Iceland") + strpos(study,"Korea") + strpos(study,"New Zealand")
gen smallflag = strpos(study,"Castiglione") + strpos(study,"Diamond Princess") + strpos(study,"Gangelt")

quietly gen ifr_trace = ifr if traceflag
separate ifr_trace if traceflag, by(study) veryshortlabel
quietly gen log_ifr_trace = log(ifr) if traceflag
separate log_ifr_trace if traceflag, by(study) veryshortlabel

quietly gen ifr_small = ifr if smallflag
separate ifr_small if smallflag, by(study) veryshortlabel
quietly gen log_ifr_small = log(ifr) if smallflag
separate log_ifr_small if smallflag, by(study) veryshortlabel

* List observations

sort study median_age
list study agegroup median_age ifr ifr_se

* Merge with metareg predictions

merge 1:1 study median_age using "Metareg_predictions.dta" 
gen exp_conf_hitop = min(exp_conf_high,21) if missing(study)
gen exp_pred_hitop = min(exp_pred_high,21) if missing(study)
sort median_age

* Make comparison chart

local bot_age = 45
local top_age = 85
twoway line expfit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	      legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) ///
	|| rarea exp_pred_low exp_pred_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(2 "Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea exp_conf_low exp_conf_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(3 "Confidence Interval")) fcolor(purple%30) lcolor(blue) /// 
	|| scatter ifr_small? median_age if inrange(median_age,`bot_age',`top_age'), ///  
	   msymbol(th sh dh) mcolor(orange navy black) ///
	|| scatter ifr_trace? median_age if inrange(median_age,`bot_age',`top_age'), ///
	      msymbol(O S D) mcolor(cranberry green blue) ///
   	   legend(pos(11) ring(0) region(lwidth(thin) fcolor(none)) ///
	     colgap(2) rowgap(2) cols(1) colfirst ///
       	 order(1 3 2 4 5 6 7 8 9)) ///
	   xlabel(45 50 55 60 65 70 75 80 85) ylabel(0 3 6 9 12 15 18 21) xtitle("Median Age",height(5)) ///
	   title("Percent", placement(ne) size(medsmall)) name(Other_comparison_chart)

capture erase metareg_predictions.dta
capture erase temp_store.dta	   
